/*
 * Copyright 2009 Arne Vajhj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.remote;

import java.io.IOException;
import java.net.Socket;
import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.record.RecordException;

/**
 * Class ClientThread is the server side of single clients connection
 * to a single remote ISAM source.
 */
public class ClientThread extends Thread {
    private static Logger log = Logger.getLogger(ClientThread.class.getName());
	private Socket cli;
	private IsamSource isam;
	private IsamProtocolServer del;
	/**
	 * Create instance of ClientThread.
	 * @param cli client socket
	 * @param isam ISAM source
	 */
	public ClientThread(Socket cli, IsamSource isam) {
		this.cli = cli;
		this.isam = isam;
	}
	@Override
	public void run() {
		try {
            int version = cli.getInputStream().read();
            switch(version) {
                case IsamProtocol.O_V1:
                    del = new ObjectServer(cli.getInputStream(), cli.getOutputStream());
                    log.finer("Using object streams for wire protocol");
                    break;
                case IsamProtocol.B_V1:
                    del = new ByteServer(cli.getInputStream(), cli.getOutputStream());
                    log.finer("Using byte streams for wire protocol");
                    break;
                default:
                    throw new IsamException("Unsupported protocol version");
            }
            del.process(isam);
			cli.close();
		} catch (IOException e) {
			e.printStackTrace(System.err);
        } catch (RecordException e) {
            e.printStackTrace(System.err);
		} catch (IsamException e) {
            e.printStackTrace(System.err);
        }
	}
}
