package dk.vajhoej.isam.test;

import org.junit.Test;

import com.tngtech.archunit.core.domain.JavaClasses;
import com.tngtech.archunit.core.importer.ClassFileImporter;

import static com.tngtech.archunit.lang.syntax.ArchRuleDefinition.classes;
import static com.tngtech.archunit.lang.syntax.ArchRuleDefinition.noClasses;
import static com.tngtech.archunit.lang.syntax.ArchRuleDefinition.methods;

public class TestArchitecture {
	private static final String MOD_PASCAL_CASE = "([A-Z]+[a-z0-9]*)+";
	private static final String MOD_CAMEL_CASE = "[a-z][a-z0-9]*([A-Z]+[a-z0-9]*)*";
	private static final String DIGITS = "[0-9]+";
	private static final String ARTIFICIAL_METHOD = "\\$.*";
	private static final String OUTER_CLZ_NAM_PAT = MOD_PASCAL_CASE;
	private static final String INNER_CLZ_NAM_PAT = "\\$(" + OUTER_CLZ_NAM_PAT + "|" + DIGITS + ")";
	private static final String CLZ_NAM_PAT = OUTER_CLZ_NAM_PAT + "(" + INNER_CLZ_NAM_PAT + ")*";
	private static final String METH_NAM_PAT = MOD_CAMEL_CASE + "|" + ARTIFICIAL_METHOD;
	private JavaClasses thislib = new ClassFileImporter().importPackages("dk.vajhoej.isam", "dk.vajhoej.bdb", "dk.vajhoej.vbisam", "dk.vajhoej.vms.rms");
	private String[] modules = { "cache", "local", "remote" };
	private String[] modules_notlocal = { "cache", "remote" };
	private String[] plugins = { "isam.dummy", "isam.emulated", "bdb", "vbisam", "vms.rms" };
	@Test
	public void testNamingConventionClasses() {
		classes().that().resideInAnyPackage("dk.vajhoej.isam").should().haveNameMatching("dk.vajhoej.isam." + CLZ_NAM_PAT).check(thislib);
		for(String module : modules) {
			classes().that().resideInAnyPackage("dk.vajhoej.isam." + module).should().haveNameMatching("dk.vajhoej.isam." + module + "." + CLZ_NAM_PAT).check(thislib);
		}
		for(String plugin : plugins) {
			classes().that().resideInAnyPackage("dk.vajhoej." + plugin).should().haveNameMatching("dk.vajhoej." + plugin + "." + CLZ_NAM_PAT).check(thislib);
		}
	}
	@Test
	public void testNamingConventionMethods() {		
		methods().that().areDeclaredInClassesThat().resideInAnyPackage("dk.vajhoej.isam").should().haveNameMatching(METH_NAM_PAT).check(thislib);;
	}
	@Test
	public void testLayers() {
		for(String plugin : plugins) {
			noClasses().that().resideInAnyPackage("dk.vajhoej.isam").should().dependOnClassesThat().resideInAnyPackage("dk.vajhoej." + plugin).check(thislib);
			for(String module : modules_notlocal) {
				noClasses().that().resideInAnyPackage("dk.vajhoej." + plugin).should().dependOnClassesThat().resideInAnyPackage("dk.vajhoej.isam." + module).check(thislib);
			}
		}
	}
}
