/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.test;

import static org.junit.Assert.*;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.isam.KeyInfoArrayCache;
import dk.vajhoej.isam.emulated.EmulatedIsamSource;
import dk.vajhoej.record.RecordException;

public class TestEmulatedIsamSource {
	private final static String DRIVER = "com.mysql.cj.jdbc.Driver";
	private final static String CONURL = "jdbc:mysql://localhost/Test";
	private final static String UN = "root";
	private final static String PW = "hemmeligt";
	@Before
	public void setup() {
		try {
			Class.forName(DRIVER);
			Connection con = DriverManager.getConnection(CONURL, UN, PW);
			Statement stmt = con.createStatement();
			stmt.executeUpdate("CREATE TABLE data (iv INTEGER NOT NULL, recblob BLOB, PRIMARY KEY(iv))");
			stmt.close();
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	@After
	public void teardown() {
		try {
			Connection con = DriverManager.getConnection(CONURL, UN, PW);
			Statement stmt = con.createStatement();
			stmt.executeUpdate("DROP TABLE data");
			stmt.close();
			con.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
    @Test
    public void testCreateRead() {
        try {
            TestCommonIsamSource.testCreateRead(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testCreateReadKey0() {
        try {
            TestCommonIsamSource.testCreateReadKey0(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
	@Test
	public void testCreateReadMulti() {
		try {
			TestCommonIsamSource.testCreateReadMulti(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadPerf() {
		try {
			TestCommonIsamSource.testReadPerf(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testCreatePerf() {
		try {
			TestCommonIsamSource.testCreatePerf(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGE() {
		try {
			TestCommonIsamSource.testReadGE(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadGT() {
		try {
			TestCommonIsamSource.testReadGT(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLE() {
		try {
			TestCommonIsamSource.testReadLE(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testReadLT() {
		try {
			TestCommonIsamSource.testReadLT(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
	@Test
	public void testError1() {
		try {
			TestCommonIsamSource.testError1(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		}
	}
    @Test
    public void testError2() {
        try {
            TestCommonIsamSource.testError2(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testUpdate() {
        try {
            TestCommonIsamSource.testUpdate(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testDelete() {
        try {
            TestCommonIsamSource.testDelete(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testLogging() {
        try {
            TestCommonIsamSource.testLogging(new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrency() {
        try {
            TestCommonIsamSource.testConcurrency(new EmulatedIsamSource(DRIVER, CONURL, UN, PW),
                                                 new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
    @Test
    public void testConcurrencyHard() {
        try {
            TestCommonIsamSource.testConcurrencyHard(new EmulatedIsamSource(DRIVER, CONURL, UN, PW),
                                                     new EmulatedIsamSource(DRIVER, CONURL, UN, PW));
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        }
    }
	@Test
	public void testAnalyzeCache() {
		try {
			IsamSource isam = new EmulatedIsamSource(DRIVER, CONURL, UN, PW);
			KeyInfoArrayCache.getInstance().reset();
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			isam.create(o);
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() < 0.001);
			for(int i = 1; i < 10; i++) {
				Data o2 = new Data();
				o2.setIv(258 + i);
				o2.setXv(123.456);
				o2.setSv("ABC     ");
				isam.create(o2);
			}
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() > 0.899);
			for(int i = 0; i < 10; i++) {
				Data o2 = isam.read(Data.class, new Key<Integer>(0, 258 + i));
				assertEquals("iv", 258 + i, o2.getIv());
			}
			assertTrue("hit rate", KeyInfoArrayCache.getInstance().getHitRate() > 0.949);
			isam.close();
		} catch (IsamException e) {
			e.printStackTrace();
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
}
